@@ -474,11 +474,13 @@ setMethod("getDesign", "unmarkedFramePCO",
     aschar1 <- as.character(formula)
     aschar2 <- as.character(formula[[2]])
     aschar3 <- as.character(formula[[2]][[2]])
+    aschar4 <- as.character(formula[[2]][[2]][[2]])
 
-    pformula <- as.formula(paste(aschar1[1], aschar1[3]))
-    omformula <- as.formula(paste(aschar2[1], aschar2[3]))
-    gamformula <- as.formula(paste(aschar3[1], aschar3[3]))
-    lamformula <- as.formula(formula[[2]][[2]][[2]])
+    iotaformula <- as.formula(paste(aschar1[1], aschar1[3]))
+    pformula <- as.formula(paste(aschar2[1], aschar2[3]))
+    omformula <- as.formula(paste(aschar3[1], aschar3[3]))
+    gamformula <- as.formula(paste(aschar4[1], aschar4[3]))
+    lamformula <- as.formula(formula[[2]][[2]][[2]][[2]])
 
     y <- getY(umf)
     M <- nrow(y)
@@ -528,6 +530,11 @@ setMethod("getDesign", "unmarkedFramePCO",
         if(is.factor(transCovs[,i]))
             transCovs[,i] <- factor(transCovs[,i]) # drop unused levels
 
+    Xiota.mf <- model.frame(iotaformula, transCovs, na.action = NULL)
+    Xiota <- model.matrix(iotaformula, Xiota.mf)
+    Xiota.offset <- as.vector(model.offset(Xiota.mf))
+    if(!is.null(Xiota.offset))
+        Xiota.offset[is.na(Xiota.offset)] <- 0
     Xp.mf <- model.frame(pformula, obsCovs, na.action = NULL)
     Xp <- model.matrix(pformula, Xp.mf)
     Xp.offset <- as.vector(model.offset(Xp.mf))
@@ -544,9 +551,9 @@ setMethod("getDesign", "unmarkedFramePCO",
     if(!is.null(Xom.offset))
         Xom.offset[is.na(Xom.offset)] <- 0
 
-    # determine if gamma and omega are scalar, vector, or matrix valued
+    # determine if gamma, omega, and iota are scalar, vector, or matrix valued
     # Runtime is much faster for scalars and vectors
-    Xgo <- cbind(Xgam, Xom)
+    Xgo <- cbind(Xgam, Xom, Xiota)
     getGOdims <- function(x) {
         xm <- matrix(x, M, T-1, byrow=TRUE)
 #        anyNA <- apply(is.na(xm), 1, any)
@@ -568,7 +575,8 @@ setMethod("getDesign", "unmarkedFramePCO",
         } else
             return("matrix")
         }
-    if(isTRUE(all.equal(gamformula,~1)) & isTRUE(all.equal(omformula, ~1)))
+    if(isTRUE(all.equal(gamformula,~1)) & isTRUE(all.equal(omformula, ~1)) & 
+      isTRUE(all.equal(iotaformula, ~1)))
         go.dims <- "scalar"
     else {
         go.dims.vec <- apply(Xgo, 2, getGOdims)
@@ -581,25 +589,26 @@ setMethod("getDesign", "unmarkedFramePCO",
     }
 
     if(na.rm)
-        out <- handleNA(umf, Xlam, Xgam, Xom, Xp,
-                        Xlam.offset, Xgam.offset, Xom.offset, Xp.offset,
-                        delta)
+        out <- handleNA(umf, Xlam, Xgam, Xom, Xp, Xiota,
+            Xlam.offset, Xgam.offset, Xom.offset, Xp.offset, Xiota.offset,
+            delta)
     else {   # delta needs to be formatted first
         ya <- array(y, c(M, J, T))
         yna <- apply(is.na(ya), c(1,3), all)
         delta <- formatDelta(delta, yna)
-        out <- list(y=y, Xlam=Xlam, Xgam=Xgam, Xom=Xom, Xp=Xp,
+        out <- list(y=y, Xlam=Xlam, Xgam=Xgam, Xom=Xom, Xp=Xp, Xiota=Xiota,
                     Xlam.offset=Xlam.offset, Xgam.offset=Xgam.offset,
-                    Xom.offset=Xom.offset, Xp.offset=Xp.offset,
-                    delta=delta, removed.sites=integer(0), go.dims=go.dims)
+                    Xom.offset=Xom.offset, Xp.offset=Xp.offset, 
+                    Xiota.offset=Xiota.offset,
+                    delta=delta, removed.sites=integer(0))
     }
 
     return(list(y = out$y, Xlam = out$Xlam, Xgam = out$Xgam,
-                Xom = out$Xom, Xp = out$Xp,
+                Xom = out$Xom, Xp = out$Xp, Xiota = out$Xiota,
                 Xlam.offset=Xlam.offset, Xgam.offset=Xgam.offset,
-                Xom.offset=Xom.offset, Xp.offset=Xp.offset,
-                delta = out$delta, removed.sites = out$removed.sites,
-                go.dims = go.dims))
+                Xom.offset=Xom.offset, Xp.offset=Xp.offset, 
+                Xiota.offset=Xiota.offset, delta = out$delta, 
+                removed.sites = out$removed.sites, go.dims = go.dims))
 })
 
 
@@ -610,8 +619,8 @@ setMethod("getDesign", "unmarkedFramePCO",
 
 
 setMethod("handleNA", "unmarkedFramePCO",
-    function(umf, Xlam, Xgam, Xom, Xp, Xlam.offset, Xgam.offset,
-             Xom.offset, Xp.offset, delta)
+    function(umf, Xlam, Xgam, Xom, Xp, Xiota, Xlam.offset, Xgam.offset,
+             Xom.offset, Xp.offset, Xiota.offset, delta)
 {
 	obsToY <- obsToY(umf)
 	if(is.null(obsToY)) stop("obsToY cannot be NULL to clean data.")
@@ -712,14 +721,19 @@ setMethod("handleNA", "unmarkedFramePCO",
                      drop = FALSE]
             Xp.offset <- Xp.offset[!sites.to.remove[rep(1:M, each = J*T)],,
                      drop = FALSE]
+            Xiota <- Xiota[!sites.to.remove[rep(1:M, each = T-1)],,
+                       drop = FALSE]
+            Xiota.offset <- Xiota.offset[!sites.to.remove[rep(1:M, each = T-1)],,
+                       drop = FALSE]
             delta <- delta[!sites.to.remove, ,drop =FALSE]
             warning(paste(num.to.remove, "sites have been discarded because of missing data."), call.=FALSE)
 	}
 
-	list(y = y.wide, Xlam = Xlam, Xgam = Xgam, Xom = Xom, Xp = Xp,
+	list(y = y.wide, Xlam = Xlam, Xgam = Xgam, Xom = Xom, Xp = Xp, Xiota = Xiota, 
              Xlam.offset=Xlam.offset, Xgam.offset=Xgam.offset,
-             Xom.offset=Xom.offset, Xp.offset=Xp.offset,
-             delta = delta, removed.sites = which(sites.to.remove))
+             Xom.offset=Xom.offset, Xp.offset=Xp.offset, 
+             Xiota.offset=Xiota.offset, delta = delta, 
+             removed.sites = which(sites.to.remove))
 })
 
 
