@@ -38,6 +38,7 @@ setClass("unmarkedMultFrame",
     contains="unmarkedFrame")
 
 
+
 ## a class for distance sampling data
 setClass("unmarkedFrameDS",
     representation(
@@ -64,6 +65,11 @@ setClass("unmarkedFrameDS",
 setClass("unmarkedFrameOccu",
 		contains = "unmarkedFrame")
 
+setClass("unmarkedFrameOccuFP",
+         representation(
+           type = "numeric"),
+         contains = "unmarkedFrame")
+
 
 setClass("unmarkedFramePCount",
 		contains = "unmarkedFrame")
@@ -77,15 +83,12 @@ setClass("unmarkedFrameMPois",
 
 
 setClass("unmarkedFrameG3",
-    representation(
-        primaryPeriod = "matrix"),
-    contains = "unmarkedMultFrame")
+         contains = "unmarkedMultFrame")
 
 
 setClass("unmarkedFramePCO",
-    representation(
-        primaryPeriod = "matrix"),
-    contains = "unmarkedMultFrame")
+         representation(primaryPeriod = "matrix"),
+         contains = "unmarkedMultFrame")
 
 
 setClass("unmarkedFrameGMM",
@@ -102,6 +105,9 @@ setClass("unmarkedFrameGDS",
         unitsIn = "character"),
     contains = "unmarkedFrameG3")
 
+setClass("unmarkedFrameGPC",
+    contains = "unmarkedFrameG3")
+
 
 
 # ------------------------------- CONSTRUCTORS ---------------------------
@@ -161,6 +167,61 @@ unmarkedFrameOccu <- function(y, siteCovs = NULL, obsCovs = NULL, mapInfo)
     umf
 }
 
+unmarkedFrameOccuFP <- function(y, siteCovs = NULL, obsCovs = NULL, type, mapInfo)
+{
+  J <- ncol(y)
+  umf <- unmarkedFrame(y, siteCovs, obsCovs, obsToY = diag(J),
+                       mapInfo = mapInfo)
+  umf <- as(umf, "unmarkedFrameOccuFP")
+  umf@type <- type
+  umf
+}
+
+
+
+
+unmarkedFramePCount <- function(y, siteCovs = NULL, obsCovs = NULL, mapInfo)
+{
+    J <- ncol(y)
+    umf <- unmarkedFrame(y, siteCovs, obsCovs, obsToY = diag(J),
+        mapInfo = mapInfo)
+    umf <- as(umf, "unmarkedFramePCount")
+    umf
+}
+
+
+
+unmarkedFrameMPois <- function(y, siteCovs = NULL, obsCovs = NULL, type,
+    obsToY, mapInfo, piFun)
+{
+    if(!missing(type)) {
+        switch(type,
+            removal = {
+                obsToY <- matrix(1, ncol(y), ncol(y))
+                obsToY[col(obsToY) < row(obsToY)] <- 0
+                #obsToY <- diag(ncol(y))
+                #obsToY[upper.tri(obsToY)] <- 1
+                piFun <- "removalPiFun"
+                },
+            double = {
+                #obsToY <- matrix(c(1, 0, 0, 1, 1, 1), 2, 3)
+                obsToY <- matrix(1, 2, 3)
+                piFun <- "doublePiFun"
+                })
+    } else {
+        if(missing(obsToY))
+            stop("obsToY is required for multinomial-Poisson data with no specified type.")
+        type <- "userDefined"
+        }
+    umf <- unmarkedFrame(y, siteCovs, obsCovs, obsToY = obsToY,
+        mapInfo = mapInfo)
+    umf <- as(umf, "unmarkedFrameMPois")
+    umf@piFun <- piFun
+    umf@samplingMethod <- type
+    umf
+}
+
+
 
 # This function constructs an unmarkedMultFrame object.
 unmarkedMultFrame <- function(y, siteCovs = NULL, obsCovs = NULL,
@@ -189,6 +250,9 @@ unmarkedMultFrame <- function(y, siteCovs = NULL, obsCovs = NULL,
 }
 
 
+
+
+
 # This function constructs an unmarkedMultFrame object.
 unmarkedFrameGMM <- function(y, siteCovs = NULL, obsCovs = NULL, numPrimary,
 	yearlySiteCovs = NULL, type, obsToY, piFun)
@@ -284,49 +348,44 @@ unmarkedFrameGDS <- function(y, siteCovs, numPrimary,
 
 
 
-unmarkedFramePCount <- function(y, siteCovs = NULL, obsCovs = NULL, mapInfo)
-{
-    J <- ncol(y)
-    umf <- unmarkedFrame(y, siteCovs, obsCovs, obsToY = diag(J),
-        mapInfo = mapInfo)
-    umf <- as(umf, "unmarkedFramePCount")
-    umf
-}
+# This function constructs an  object.
+unmarkedFrameGPC <- function(y, siteCovs=NULL, obsCovs=NULL, numPrimary,
+                             yearlySiteCovs=NULL) {
+    if(numPrimary < 2)
+        stop("numPrimary must be >1. Use pcount of numPrimary=1")
+    J <- ncol(y) / numPrimary
+    obsToY <- diag(J*numPrimary)
+    if(missing(siteCovs))
+        siteCovs <- NULL
 
-
-
-unmarkedFrameMPois <- function(y, siteCovs = NULL, obsCovs = NULL, type,
-    obsToY, mapInfo, piFun)
-{
-    if(!missing(type)) {
-        switch(type,
-            removal = {
-                obsToY <- matrix(1, ncol(y), ncol(y))
-                obsToY[col(obsToY) < row(obsToY)] <- 0
-                #obsToY <- diag(ncol(y))
-                #obsToY[upper.tri(obsToY)] <- 1
-                piFun <- "removalPiFun"
-                },
-            double = {
-                #obsToY <- matrix(c(1, 0, 0, 1, 1, 1), 2, 3)
-                obsToY <- matrix(1, 2, 3)
-                piFun <- "doublePiFun"
-                })
-    } else {
-        if(missing(obsToY))
-            stop("obsToY is required for multinomial-Poisson data with no specified type.")
-        type <- "userDefined"
+    umf <- unmarkedFrame(y = y, siteCovs = siteCovs, obsCovs = obsCovs,
+                         obsToY = obsToY)
+    umf <- as(umf, "unmarkedMultFrame")
+    umf@numPrimary <- numPrimary
+    if(missing(yearlySiteCovs))
+        yearlySiteCovs <- NULL
+    if(class(yearlySiteCovs) == "list") {
+        yearlySiteVars <- names(yearlySiteCovs)
+        for(i in seq(length(yearlySiteVars))) {
+            if(!(class(yearlySiteCovs[[i]]) %in% c("matrix","data.frame")))
+                stop("At least one element of yearlySiteCovs is not a matrix or data frame.")
+            if(ncol(yearlySiteCovs[[i]]) != numPrimary |
+                nrow(yearlySiteCovs[[i]]) != nrow(y))
+                    stop("At least one matrix in yearlySiteCovs has incorrect number of dimensions.")
+            }
+        yearlySiteCovs <- data.frame(lapply(yearlySiteCovs, function(x)
+            as.vector(t(x))))
         }
-    umf <- unmarkedFrame(y, siteCovs, obsCovs, obsToY = obsToY,
-        mapInfo = mapInfo)
-    umf <- as(umf, "unmarkedFrameMPois")
-    umf@piFun <- piFun
-    umf@samplingMethod <- type
+
+    umf@yearlySiteCovs <- yearlySiteCovs
+    umf <- as(umf, "unmarkedFrameGPC")
     umf
 }
 
 
 
+
+
 unmarkedFramePCO <- function(y, siteCovs = NULL, obsCovs = NULL,
     yearlySiteCovs = NULL, mapInfo, numPrimary, primaryPeriod)
 {
@@ -438,6 +497,7 @@ setMethod("yearlySiteCovs", "unmarkedMultFrame", function(object) {
     return(object@yearlySiteCovs)
 })
 
+
 setGeneric("obsCovs", function(object,...) standardGeneric("obsCovs"))
 setMethod("obsCovs", "unmarkedFrame", function(object, matrices = FALSE) {
     M <- numSites(object)
@@ -497,7 +557,6 @@ setReplaceMethod("yearlySiteCovs", "unmarkedMultFrame",
         object
     })
 
-
 setGeneric("obsToY<-", function(object, value) standardGeneric("obsToY<-"))
 setReplaceMethod("obsToY", "unmarkedFrame", function(object, value) {
     object@obsToY <- value
@@ -685,6 +744,7 @@ setMethod("[", c("unmarkedFrame", "numeric", "missing", "missing"),
 
 
 ## remove obs only
+### RBC: Why??? this doesn't allow umf[,c(1,1)]
 setMethod("[", c("unmarkedFrame", "missing", "numeric", "missing"),
 		function(x, i, j)
 {
@@ -782,6 +842,7 @@ setMethod("[", c("unmarkedMultFrame", "missing", "numeric", "missing"),
 })
 
 
+
 ## for multframes, must remove years at a time
 setMethod("[", c("unmarkedMultFrame", "numeric", "missing", "missing"),
 		function(x, i, j)
@@ -828,6 +889,7 @@ setMethod("[", c("unmarkedMultFrame", "numeric", "missing", "missing"),
 
 
 
+
 setMethod("[", c("unmarkedFrameGMM", "numeric", "missing", "missing"),
 		function(x, i, j)
 {
@@ -836,10 +898,29 @@ setMethod("[", c("unmarkedFrameGMM", "numeric", "missing", "missing"),
                      yearlySiteCovs=yearlySiteCovs(multf),
                      obsCovs=obsCovs(multf),
                      piFun=x@piFun, type=x@samplingMethod,
-                     obsToY=x@obsToY, numPrimary=x@numPrimary)
+                     obsToY=multf@obsToY, numPrimary=multf@numPrimary)
 })
 
 
+setMethod("[", c("unmarkedFrameGPC", "numeric", "missing", "missing"),
+		function(x, i, j)
+{
+    multf <- callNextMethod(x, i, j) # unmarkedMultFrame
+    class(multf) <- "unmarkedFrameGPC"
+    multf
+})
+
+
+setMethod("[", c("unmarkedFrameGPC", "missing", "numeric", "missing"),
+		function(x, i, j)
+{
+    multf <- as(x, "unmarkedMultFrame")
+    out <- callNextMethod(multf, i, j) # unmarkedMultFrame
+    as(out, "unmarkedFrameGPC")
+})
+
+
+
 
 
 setMethod("[", c("unmarkedFrameGDS", "numeric", "missing", "missing"),
@@ -878,6 +959,18 @@ setMethod("[", c("unmarkedFramePCO", "numeric", "missing", "missing"),
 })
 
 
+setMethod("[", c("unmarkedFramePCO", "missing", "numeric", "missing"),
+		function(x, i, j)
+{
+    multf <- callNextMethod(x, i, j) # unmarkedMultFrame
+    unmarkedFramePCO(y=getY(multf), siteCovs=siteCovs(multf),
+                     yearlySiteCovs=yearlySiteCovs(multf),
+                     obsCovs=obsCovs(multf),
+                     numPrimary=length(j),
+                     primaryPeriod=x@primaryPeriod[,j,drop=FALSE])
+})
+
+
 
 
 
